# import argparse
# import os
# import sys


# parser = argparse.ArgumentParser(
#     description="CLI for interacting with the Arc Web Framework", prog="arc")


# parser.add_argument(
#     "Data",
#     metavar="data",
#     type=str,
#     help="test"
# )

# args = parser.parse_args()

# data = args.Data

# print("Repeated" + data)
